SuperStrict

' mbcc22 by Holzchopf

' the modules we need
Framework BRL.GLMax2D
Import BRL.PNGLoader
Import BRL.Timer
Import BRL.Random

' the consts we like
Const STATE_LOADING%	= 0
Const STATE_EXPLODING%	= 1
Const STATE_IDLE%		= 2

' the code we want
' (seriously, who doesn't want OGL?)
SetGraphicsDriver GLMax2DDriver()
' (... or non-windowed games at any other resolution than 800x600px ?!)
Graphics(800,600)

' If you're not familiar to BlitzMax, let me explain the next line of code:
' Local tells BlitzMax to define a variable, which is only locally visible.
' timer is the name of that variable
' :TTimer defines the data type, ":" is only a delimeter, the type name
' itself is TTimer.
' Then, this newly defined variable gets declared a value:
' CreateTimer(50) is a function that returns an object of type TTimer and
' thus perfectly fits into our "timer" variable, so the value of "timer"
' is now the TTimer object, that was created using CreateTimer(50).
' Oh, and for the necessity of a timer:
' A timer "ticks" n times per second - we can listen to those ticks and
' even wait for them. Waiting doesn't cost much CPU-Power, so this is a
' gentle way to slow down the application, namely the main loop, so it
' only runs at a framerate of 50Hz. (Don't forget WaitTimer(timer) !)
Local timer:TTimer = CreateTimer(50)

Local imgc:TImage = LoadImage("img_c.png")	' coloured image
Local imgg:TImage = LoadImage("img_g.png")	' grey scale image
' since imgc will be drawn by the "special" function, which centers the
' image automatically, only imgg has to be midhandled
MidHandleImage(imgg)

' all state variables to simulate loading and controlling all the
' different display / app states
Local state% = STATE_LOADING
Local progress%, progress_max% = 100
Local progress_tm%
Local explo%, explo_frames% = 5, explo_perframe% = 20
' beams for explosion -fx
Local sfbeams:TScreenFillingBeam[explo_frames *explo_perframe]
' initialise screen filling beams
For Local f% = 0 Until explo_frames
	For Local e% = 0 Until explo_perframe
		sfbeams[f *explo_perframe +e] = New TScreenFillingBeam.Randomise()
	Next
Next

' main loop
While Not (KeyDown(KEY_ESCAPE) Or AppTerminate())
	' compute
	Select state
	' loading process
	Case STATE_LOADING
		' update progress (in arbitrary intervals)
		If MilliSecs() >= progress_tm Then
			progress :+ 1
			progress_tm = MilliSecs() +Rand(20, 200)
		EndIf
		' if done, go to next state
		If progress = progress_max Then
			state = STATE_EXPLODING
		EndIf
	' explosion, used as transition from loading to idle
	Case STATE_EXPLODING
		' intensify explosion
		explo :+ 1
		' until done, then go to idle state
		If explo > explo_frames Then
			state = STATE_IDLE
			SetClsColor(255,255,255)
		EndIf
	' idle
	Case STATE_IDLE
		' end on input
		If GetChar() > 0 Then
			End
		EndIf
	End Select

	' display
	Cls
	SetBlend ALPHABLEND
	SetAlpha 1
	If state < STATE_IDLE Then
		' draw coloured image on top of grey scale image
		DrawImage(imgg, 400,300)
		DrawImageDeg(imgc, 400,300, 360.0 *progress /progress_max)
		' on top of that: draw explosions (consisting of beams)
		If state = STATE_EXPLODING Then
			SetBlend LIGHTBLEND
			SetAlpha 0.5
			For Local f% = 0 Until explo
				For Local e% = 0 Until explo_perframe
					sfbeams[f *explo_perframe +e].Draw()
				Next
			Next
		EndIf
	Else
		SetColor(0,0,0)
		DrawText("Done. Press any key to quit.", 10, 10)
	EndIf
	
	' show drawn screen buffer
	Flip 0
	WaitTimer(timer)
Wend
End

' All hail to the expressive type name!
Type TScreenFillingBeam
	' coordinates for polygon
	Field pts#[6]
	Method Randomise:TScreenFillingBeam()
		Local as% = Rand(0, 359)
		Local ad% = Rand(5, 15)
		' first point is always middle of screen
		pts[0] = 400
		pts[1] = 300
		' the other two not, apparently
		Local x#, y#
		For Local p% = 0 To 1
			x = Cos(as+ad*p) *500
			y = Sin(as+ad*p) *500
			pts[p*2+2] = 400 +x
			pts[p*2+3] = 300 +y
		Next
		Return Self
	End Method
	Method Draw()
		DrawPoly(pts)
	End Method
End Type

Function DrawImageDeg(pImg:TImage, pX:Int, pY:Int, pDeg:Int)
	Local iframe:TImageFrame = pImg.Frame(0)
	If iframe Then
		' dirty hack!
		' necessary to set state_boundtex
		' also, this enables GL_TEXTURE_2D and sets the texture
		iframe.Draw(0,0 ,0,0 ,0,0 ,0,0 ,0,0)
		
		' image size
		Local w% = pImg.width
		Local h% = pImg.height
		
		' tex scaling
		Local us# = TGLImageFrame(iframe).u1
		Local vs# = TGLImageFrame(iframe).v1
		
		' how many pieces (of 45) to draw
		Local pcs% = pDeg / 45
		' how much of last piece to draw
		Local deg% = pDeg Mod 45
		
		glBegin GL_TRIANGLES
		For Local p% = 0 To pcs
			Local az#
			' full piece
			If p < pcs
				az = 45
			' chopped piece
			Else
				az = deg
			EndIf
			' all the coordinates
			Local sx#, sy#
			Local ex#, ey#
			' starting angle of piece
			Local a% = p *45 -90	' starting upwards, thus -90
			sx = Cos(a)
			sy = Sin(a)
			ex = Cos(a+az)
			ey = Sin(a+az)
			' restrict to -0.5 ... 0.5
			Clamp(sx, sy)
			Clamp(ex, ey)
			' draw the piece with textured triangles
			glTexCoord2f 0.5*us, 0.5*vs
			glVertex2d pX, pY
			glTexCoord2f (0.5+sx)*us, (0.5+sy)*vs
			glVertex2d pX+sx*w, pY+sy*h
			glTexCoord2f (0.5+ex)*us, (0.5+ey)*vs
			glVertex2d pX+ex*w, pY+ey*h
		Next
		glEnd
	EndIf
End Function

' documentaion unnecessary, function does what name implies ;-)

' Clamps the vector given by pX,pY to a square field from -0.5 to +0.5
' in x and y respectively. Thus, the length of the vector is _not_
' unified, but instead, the vector won't exceed the mentioned absolute
' coordinates.
Function Clamp(pX# Var, pY# Var)
	' scale factors in x, y and the d
	Local fx#=1, fy#=1, f#
	' scale down in x/y
	If Abs(pX) > 0.5 Then
		fx = 0.5 /Abs(pX)
	EndIf
	If Abs(pY) > 0.5 Then
		fy = 0.5 /Abs(pY)
	EndIf
	' find definitive scaling factor (the one axis, that needs more
	' scaling, defines the factor)
	f = Min(fx, fy)
	' scale
	pX :* f
	pY :* f
End Function

